@@ -13,12 +13,14 @@ const PREDICATE_STEP_TYPE_STRING = 2;
 
 const LANGUAGE_FUNCTION_REGEX = /^_?tree_sitter_\w+/;
 
-var VERSION;
-var MIN_COMPATIBLE_VERSION;
-var TRANSFER_BUFFER;
-var currentParseCallback;
-var currentLogCallback;
+let VERSION;
+let MIN_COMPATIBLE_VERSION;
+let TRANSFER_BUFFER;
+let currentParseCallback;
+// eslint-disable-next-line no-unused-vars
+let currentLogCallback;
 
+// eslint-disable-next-line no-unused-vars
 class ParserImpl {
   static init() {
     TRANSFER_BUFFER = C._ts_init();
@@ -50,7 +52,7 @@ class ParserImpl {
       if (version < MIN_COMPATIBLE_VERSION || VERSION < version) {
         throw new Error(
           `Incompatible language version ${version}. ` +
-          `Compatibility range ${MIN_COMPATIBLE_VERSION} through ${VERSION}.`
+          `Compatibility range ${MIN_COMPATIBLE_VERSION} through ${VERSION}.`,
         );
       }
     } else {
@@ -62,16 +64,16 @@ class ParserImpl {
   }
 
   getLanguage() {
-    return this.language
+    return this.language;
   }
 
   parse(callback, oldTree, options) {
     if (typeof callback === 'string') {
-      currentParseCallback = (index, _, endIndex) => callback.slice(index, endIndex);
+      currentParseCallback = (index, _) => callback.slice(index);
     } else if (typeof callback === 'function') {
       currentParseCallback = callback;
     } else {
-      throw new Error("Argument must be a string or a function");
+      throw new Error('Argument must be a string or a function');
     }
 
     if (this.logCallback) {
@@ -99,7 +101,7 @@ class ParserImpl {
       this[1],
       oldTree ? oldTree[0] : 0,
       rangeAddress,
-      rangeCount
+      rangeCount,
     );
 
     if (!treeAddress) {
@@ -118,19 +120,33 @@ class ParserImpl {
     C._ts_parser_reset(this[0]);
   }
 
-  setTimeoutMicros(timeout) {
-    C._ts_parser_set_timeout_micros(this[0], timeout);
+  getIncludedRanges() {
+    const count = C._malloc(SIZE_OF_INT);
+    const rangeAddress = C._ts_parser_included_ranges(this[0], count);
+    count = getValue(count, 'i32');
+    const result = new Array(count);
+    if (count > 0) {
+      let address = rangeAddress;
+      for (let i = 0; i < count; i++) {
+        result[i] = unmarshalRange(address);
+        address += SIZE_OF_RANGE;
+      }
+    }
   }
 
   getTimeoutMicros() {
     return C._ts_parser_timeout_micros(this[0]);
   }
 
+  setTimeoutMicros(timeout) {
+    C._ts_parser_set_timeout_micros(this[0], timeout);
+  }
+
   setLogger(callback) {
     if (!callback) {
       callback = null;
-    } else if (typeof callback !== "function") {
-      throw new Error("Logger callback must be a function");
+    } else if (typeof callback !== 'function') {
+      throw new Error('Logger callback must be a function');
     }
     this.logCallback = callback;
     return this;
@@ -169,6 +185,14 @@ class Tree {
     return unmarshalNode(this);
   }
 
+  rootNodeWithOffset(offsetBytes, offsetExtent) {
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    setValue(address, offsetBytes, 'i32');
+    marshalPoint(address + SIZE_OF_INT, offsetExtent);
+    C._ts_tree_root_node_with_offset_wasm(this[0]);
+    return unmarshalNode(this);
+  }
+
   getLanguage() {
     return this.language;
   }
@@ -196,6 +220,22 @@ class Tree {
     }
     return result;
   }
+
+  getIncludedRanges() {
+    C._ts_tree_included_ranges_wasm(this[0]);
+    const count = getValue(TRANSFER_BUFFER, 'i32');
+    const buffer = getValue(TRANSFER_BUFFER + SIZE_OF_INT, 'i32');
+    const result = new Array(count);
+    if (count > 0) {
+      let address = buffer;
+      for (let i = 0; i < count; i++) {
+        result[i] = unmarshalRange(address);
+        address += SIZE_OF_RANGE;
+      }
+      C._free(buffer);
+    }
+    return result;
+  }
 }
 
 class Node {
@@ -247,31 +287,36 @@ class Node {
     return C._ts_node_next_parse_state_wasm(this.tree[0]);
   }
 
-  isNamed() {
+  get isNamed() {
     marshalNode(this);
     return C._ts_node_is_named_wasm(this.tree[0]) === 1;
   }
 
-  hasError() {
+  get hasError() {
     marshalNode(this);
     return C._ts_node_has_error_wasm(this.tree[0]) === 1;
   }
 
-  hasChanges() {
+  get hasChanges() {
     marshalNode(this);
     return C._ts_node_has_changes_wasm(this.tree[0]) === 1;
   }
 
-  isError() {
+  get isError() {
     marshalNode(this);
     return C._ts_node_is_error_wasm(this.tree[0]) === 1;
   }
 
-  isMissing() {
+  get isMissing() {
     marshalNode(this);
     return C._ts_node_is_missing_wasm(this.tree[0]) === 1;
   }
 
+  get isExtra() {
+    marshalNode(this);
+    return C._ts_node_is_extra_wasm(this.tree[0]) === 1;
+  }
+
   equals(other) {
     return this.id === other.id;
   }
@@ -282,17 +327,6 @@ class Node {
     return unmarshalNode(this.tree);
   }
 
-  fieldNameForChild(index) {
-    marshalNode(this);
-    const address = C._ts_node_field_name_for_child_wasm(this.tree[0], index);
-    if (!address) {
-      return null;
-    }
-    const result = AsciiToString(address);
-    // must not free, the string memory is owned by the language
-    return result;
-  }
-
   namedChild(index) {
     marshalNode(this);
     C._ts_node_named_child_wasm(this.tree[0], index);
@@ -310,6 +344,55 @@ class Node {
     if (fieldId !== -1) return this.childForFieldId(fieldId);
   }
 
+  fieldNameForChild(index) {
+    marshalNode(this);
+    const address = C._ts_node_field_name_for_child_wasm(this.tree[0], index);
+    if (!address) {
+      return null;
+    }
+    const result = AsciiToString(address);
+    // must not free, the string memory is owned by the language
+    return result;
+  }
+
+  childrenForFieldName(fieldName) {
+    const fieldId = this.tree.language.fields.indexOf(fieldName);
+    if (fieldId !== -1 && fieldId !== 0) return this.childrenForFieldId(fieldId);
+  }
+
+  childrenForFieldId(fieldId) {
+    marshalNode(this);
+    C._ts_node_children_by_field_id_wasm(this.tree[0], fieldId);
+    const count = getValue(TRANSFER_BUFFER, 'i32');
+    const buffer = getValue(TRANSFER_BUFFER + SIZE_OF_INT, 'i32');
+    const result = new Array(count);
+    if (count > 0) {
+      let address = buffer;
+      for (let i = 0; i < count; i++) {
+        result[i] = unmarshalNode(this.tree, address);
+        address += SIZE_OF_NODE;
+      }
+      C._free(buffer);
+    }
+    return result;
+  }
+
+  firstChildForIndex(index) {
+    marshalNode(this);
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    setValue(address, index, 'i32');
+    C._ts_node_first_child_for_byte_wasm(this.tree[0]);
+    return unmarshalNode(this.tree);
+  }
+
+  firstNamedChildForIndex(index) {
+    marshalNode(this);
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    setValue(address, index, 'i32');
+    C._ts_node_first_named_child_for_byte_wasm(this.tree[0]);
+    return unmarshalNode(this.tree);
+  }
+
   get childCount() {
     marshalNode(this);
     return C._ts_node_child_count_wasm(this.tree[0]);
@@ -403,7 +486,7 @@ class Node {
       startPosition.row,
       startPosition.column,
       endPosition.row,
-      endPosition.column
+      endPosition.column,
     );
 
     // Instantiate the nodes based on the data returned.
@@ -448,6 +531,11 @@ class Node {
     return unmarshalNode(this.tree);
   }
 
+  get descendantCount() {
+    marshalNode(this);
+    return C._ts_node_descendant_count_wasm(this.tree[0]);
+  }
+
   get parent() {
     marshalNode(this);
     C._ts_node_parent_wasm(this.tree[0]);
@@ -460,7 +548,7 @@ class Node {
     }
 
     marshalNode(this);
-    let address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
     setValue(address, start, 'i32');
     setValue(address + SIZE_OF_INT, end, 'i32');
     C._ts_node_descendant_for_index_wasm(this.tree[0]);
@@ -473,7 +561,7 @@ class Node {
     }
 
     marshalNode(this);
-    let address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
     setValue(address, start, 'i32');
     setValue(address + SIZE_OF_INT, end, 'i32');
     C._ts_node_named_descendant_for_index_wasm(this.tree[0]);
@@ -486,7 +574,7 @@ class Node {
     }
 
     marshalNode(this);
-    let address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
     marshalPoint(address, start);
     marshalPoint(address + SIZE_OF_POINT, end);
     C._ts_node_descendant_for_position_wasm(this.tree[0]);
@@ -499,7 +587,7 @@ class Node {
     }
 
     marshalNode(this);
-    let address = TRANSFER_BUFFER + SIZE_OF_NODE;
+    const address = TRANSFER_BUFFER + SIZE_OF_NODE;
     marshalPoint(address, start);
     marshalPoint(address + SIZE_OF_POINT, end);
     C._ts_node_named_descendant_for_position_wasm(this.tree[0]);
@@ -606,19 +694,29 @@ class TreeCursor {
     return C._ts_tree_cursor_end_index_wasm(this.tree[0]);
   }
 
-  currentNode() {
+  get currentNode() {
     marshalTreeCursor(this);
     C._ts_tree_cursor_current_node_wasm(this.tree[0]);
     return unmarshalNode(this.tree);
   }
 
-  currentFieldId() {
+  get currentFieldId() {
     marshalTreeCursor(this);
     return C._ts_tree_cursor_current_field_id_wasm(this.tree[0]);
   }
 
-  currentFieldName() {
-    return this.tree.language.fields[this.currentFieldId()];
+  get currentFieldName() {
+    return this.tree.language.fields[this.currentFieldId];
+  }
+
+  get currentDepth() {
+    marshalTreeCursor(this);
+    return C._ts_tree_cursor_current_depth_wasm(this.tree[0]);
+  }
+
+  get currentDescendantIndex() {
+    marshalTreeCursor(this);
+    return C._ts_tree_cursor_current_descendant_index_wasm(this.tree[0]);
   }
 
   gotoFirstChild() {
@@ -635,6 +733,22 @@ class TreeCursor {
     return result === 1;
   }
 
+  gotoFirstChildForIndex(goalIndex) {
+    marshalTreeCursor(this);
+    setValue(TRANSFER_BUFFER + SIZE_OF_CURSOR, goalIndex, 'i32');
+    const result = C._ts_tree_cursor_goto_first_child_for_index_wasm(this.tree[0]);
+    unmarshalTreeCursor(this);
+    return result === 1;
+  }
+
+  gotoFirstChildForPosition(goalPosition) {
+    marshalTreeCursor(this);
+    marshalPoint(TRANSFER_BUFFER + SIZE_OF_CURSOR, goalPosition);
+    const result = C._ts_tree_cursor_goto_first_child_for_position_wasm(this.tree[0]);
+    unmarshalTreeCursor(this);
+    return result === 1;
+  }
+
   gotoNextSibling() {
     marshalTreeCursor(this);
     const result = C._ts_tree_cursor_goto_next_sibling_wasm(this.tree[0]);
@@ -649,6 +763,12 @@ class TreeCursor {
     return result === 1;
   }
 
+  gotoDescendant(goalDescendantindex) {
+    marshalTreeCursor(this);
+    C._ts_tree_cursor_goto_descendant_wasm(this.tree[0], goalDescendantindex);
+    unmarshalTreeCursor(this);
+  }
+
   gotoParent() {
     marshalTreeCursor(this);
     const result = C._ts_tree_cursor_goto_parent_wasm(this.tree[0]);
@@ -747,7 +867,7 @@ class Language {
       sourceAddress,
       sourceLength,
       TRANSFER_BUFFER,
-      TRANSFER_BUFFER + SIZE_OF_INT
+      TRANSFER_BUFFER + SIZE_OF_INT,
     );
 
     if (!address) {
@@ -769,11 +889,11 @@ class Language {
           break;
         case 5:
           error = new TypeError(`Bad pattern structure at offset ${errorIndex}: '${suffix}'...`);
-          word = "";
+          word = '';
           break;
         default:
           error = new SyntaxError(`Bad syntax at offset ${errorIndex}: '${suffix}'...`);
-          word = "";
+          word = '';
           break;
       }
       error.index = errorIndex;
@@ -792,7 +912,7 @@ class Language {
       const nameAddress = C._ts_query_capture_name_for_id(
         address,
         i,
-        TRANSFER_BUFFER
+        TRANSFER_BUFFER,
       );
       const nameLength = getValue(TRANSFER_BUFFER, 'i32');
       captureNames[i] = UTF8ToString(nameAddress, nameLength);
@@ -802,7 +922,7 @@ class Language {
       const valueAddress = C._ts_query_string_value_for_id(
         address,
         i,
-        TRANSFER_BUFFER
+        TRANSFER_BUFFER,
       );
       const nameLength = getValue(TRANSFER_BUFFER, 'i32');
       stringValues[i] = UTF8ToString(valueAddress, nameLength);
@@ -813,11 +933,12 @@ class Language {
     const refutedProperties = new Array(patternCount);
     const predicates = new Array(patternCount);
     const textPredicates = new Array(patternCount);
+
     for (let i = 0; i < patternCount; i++) {
       const predicatesAddress = C._ts_query_predicates_for_pattern(
         address,
         i,
-        TRANSFER_BUFFER
+        TRANSFER_BUFFER,
       );
       const stepCount = getValue(TRANSFER_BUFFER, 'i32');
 
@@ -849,103 +970,121 @@ class Language {
               isPositive = false;
             case 'any-eq?':
             case 'eq?':
-              if (steps.length !== 3) throw new Error(
-                `Wrong number of arguments to \`#${operator}\` predicate. Expected 2, got ${steps.length - 1}`
-              );
-              if (steps[1].type !== 'capture') throw new Error(
-                `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}"`
-              );
+              if (steps.length !== 3) {
+                throw new Error(
+                  `Wrong number of arguments to \`#${operator}\` predicate. Expected 2, got ${steps.length - 1}`,
+                );
+              }
+              if (steps[1].type !== 'capture') {
+                throw new Error(
+                  `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}"`,
+                );
+              }
               matchAll = !operator.startsWith('any-');
               if (steps[2].type === 'capture') {
                 const captureName1 = steps[1].name;
                 const captureName2 = steps[2].name;
-                textPredicates[i].push(function (captures) {
-                  let nodes_1 = [];
-                  let nodes_2 = [];
+                textPredicates[i].push(function(captures) {
+                  const nodes1 = [];
+                  const nodes2 = [];
                   for (const c of captures) {
-                    if (c.name === captureName1) nodes_1.push(c.node);
-                    if (c.name === captureName2) nodes_2.push(c.node);
+                    if (c.name === captureName1) nodes1.push(c.node);
+                    if (c.name === captureName2) nodes2.push(c.node);
                   }
-                  let compare = (n1, n2, positive) => {
+                  const compare = (n1, n2, positive) => {
                     return positive ?
                       n1.text === n2.text :
                       n1.text !== n2.text;
                   };
-                  return matchAll
-                    ? nodes_1.every(n1 => nodes_2.some(n2 => compare(n1, n2, isPositive)))
-                    : nodes_1.some(n1 => nodes_2.some(n2 => compare(n1, n2, isPositive)));
+                  return matchAll ?
+                    nodes1.every((n1) => nodes2.some((n2) => compare(n1, n2, isPositive))) :
+                    nodes1.some((n1) => nodes2.some((n2) => compare(n1, n2, isPositive)));
                 });
               } else {
                 captureName = steps[1].name;
                 const stringValue = steps[2].value;
-                let matches = (n) => n.text === stringValue;
-                let doesNotMatch = (n) => n.text !== stringValue;
-                textPredicates[i].push(function (captures) {
-                  let nodes = [];
+                const matches = (n) => n.text === stringValue;
+                const doesNotMatch = (n) => n.text !== stringValue;
+                textPredicates[i].push(function(captures) {
+                  const nodes = [];
                   for (const c of captures) {
                     if (c.name === captureName) nodes.push(c.node);
                   }
-                  let test = isPositive ? matches : doesNotMatch;
-                  return matchAll
-                    ? nodes.every(test)
-                    : nodes.some(test);
+                  const test = isPositive ? matches : doesNotMatch;
+                  return matchAll ?
+                    nodes.every(test) :
+                    nodes.some(test);
                 });
               }
               break;
 
-            case "any-not-match?":
-            case "not-match?":
-                isPositive = false;
-            case "any-match?":
-            case "match?":
-              if (steps.length !== 3) throw new Error(
-                `Wrong number of arguments to \`#${operator}\` predicate. Expected 2, got ${steps.length - 1}.`
-              );
-              if (steps[1].type !== 'capture') throw new Error(
-                `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}".`
-              );
-              if (steps[2].type !== 'string') throw new Error(
-                `Second argument of \`#${operator}\` predicate must be a string. Got @${steps[2].value}.`
-              );
+            case 'any-not-match?':
+            case 'not-match?':
+              isPositive = false;
+            case 'any-match?':
+            case 'match?':
+              if (steps.length !== 3) {
+                throw new Error(
+                  `Wrong number of arguments to \`#${operator}\` predicate. Expected 2, got ${steps.length - 1}.`,
+                );
+              }
+              if (steps[1].type !== 'capture') {
+                throw new Error(
+                  `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}".`,
+                );
+              }
+              if (steps[2].type !== 'string') {
+                throw new Error(
+                  `Second argument of \`#${operator}\` predicate must be a string. Got @${steps[2].value}.`,
+                );
+              }
               captureName = steps[1].name;
               const regex = new RegExp(steps[2].value);
               matchAll = !operator.startsWith('any-');
-              textPredicates[i].push(function (captures) {
+              textPredicates[i].push(function(captures) {
                 const nodes = [];
                 for (const c of captures) {
                   if (c.name === captureName) nodes.push(c.node.text);
                 }
-                let test = (text, positive) => {
+                const test = (text, positive) => {
                   return positive ?
                     regex.test(text) :
                     !regex.test(text);
                 };
                 if (nodes.length === 0) return !isPositive;
-                return matchAll
-                  ? nodes.every(text => test(text, isPositive))
-                  : nodes.some(text => test(text, isPositive))
+                return matchAll ?
+                  nodes.every((text) => test(text, isPositive)) :
+                  nodes.some((text) => test(text, isPositive));
               });
               break;
 
             case 'set!':
-              if (steps.length < 2 || steps.length > 3) throw new Error(
-                `Wrong number of arguments to \`#set!\` predicate. Expected 1 or 2. Got ${steps.length - 1}.`
-              );
-              if (steps.some(s => s.type !== 'string')) throw new Error(
-                `Arguments to \`#set!\` predicate must be a strings.".`
-              );
+              if (steps.length < 2 || steps.length > 3) {
+                throw new Error(
+                  `Wrong number of arguments to \`#set!\` predicate. Expected 1 or 2. Got ${steps.length - 1}.`,
+                );
+              }
+              if (steps.some((s) => s.type !== 'string')) {
+                throw new Error(
+                  `Arguments to \`#set!\` predicate must be a strings.".`,
+                );
+              }
               if (!setProperties[i]) setProperties[i] = {};
               setProperties[i][steps[1].value] = steps[2] ? steps[2].value : null;
               break;
 
             case 'is?':
             case 'is-not?':
-              if (steps.length < 2 || steps.length > 3) throw new Error(
-                `Wrong number of arguments to \`#${operator}\` predicate. Expected 1 or 2. Got ${steps.length - 1}.`
-              );
-              if (steps.some(s => s.type !== 'string')) throw new Error(
-                `Arguments to \`#${operator}\` predicate must be a strings.".`
-              );
+              if (steps.length < 2 || steps.length > 3) {
+                throw new Error(
+                  `Wrong number of arguments to \`#${operator}\` predicate. Expected 1 or 2. Got ${steps.length - 1}.`,
+                );
+              }
+              if (steps.some((s) => s.type !== 'string')) {
+                throw new Error(
+                  `Arguments to \`#${operator}\` predicate must be a strings.".`,
+                );
+              }
               const properties = operator === 'is?' ? assertedProperties : refutedProperties;
               if (!properties[i]) properties[i] = {};
               properties[i][steps[1].value] = steps[2] ? steps[2].value : null;
@@ -954,26 +1093,32 @@ class Language {
             case 'not-any-of?':
               isPositive = false;
             case 'any-of?':
-              if (steps.length < 2) throw new Error(
-                `Wrong number of arguments to \`#${operator}\` predicate. Expected at least 1. Got ${steps.length - 1}.`
-              );
-              if (steps[1].type !== 'capture') throw new Error(
-                `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}".`
-              );
-              for (let i = 2; i < steps.length; i++) {
-                if (steps[i].type !== 'string') throw new Error(
-                  `Arguments to \`#${operator}\` predicate must be a strings.".`
+              if (steps.length < 2) {
+                throw new Error(
+                  `Wrong number of arguments to \`#${operator}\` predicate. Expected at least 1. Got ${steps.length - 1}.`,
                 );
               }
+              if (steps[1].type !== 'capture') {
+                throw new Error(
+                  `First argument of \`#${operator}\` predicate must be a capture. Got "${steps[1].value}".`,
+                );
+              }
+              for (let i = 2; i < steps.length; i++) {
+                if (steps[i].type !== 'string') {
+                  throw new Error(
+                    `Arguments to \`#${operator}\` predicate must be a strings.".`,
+                  );
+                }
+              }
               captureName = steps[1].name;
-              const values = steps.slice(2).map(s => s.value);
-              textPredicates[i].push(function (captures) {
+              const values = steps.slice(2).map((s) => s.value);
+              textPredicates[i].push(function(captures) {
                 const nodes = [];
                 for (const c of captures) {
                   if (c.name === captureName) nodes.push(c.node.text);
                 }
                 if (nodes.length === 0) return !isPositive;
-                return nodes.every(text => values.includes(text)) === isPositive;
+                return nodes.every((text) => values.includes(text)) === isPositive;
               });
               break;
 
@@ -999,7 +1144,7 @@ class Language {
       predicates,
       Object.freeze(setProperties),
       Object.freeze(assertedProperties),
-      Object.freeze(refutedProperties)
+      Object.freeze(refutedProperties),
     );
   }
 
@@ -1018,28 +1163,28 @@ class Language {
         bytes = Promise.resolve(fs.readFileSync(url));
       } else {
         bytes = fetch(url)
-          .then(response => response.arrayBuffer()
-            .then(buffer => {
+          .then((response) => response.arrayBuffer()
+            .then((buffer) => {
               if (response.ok) {
                 return new Uint8Array(buffer);
               } else {
                 const body = new TextDecoder('utf-8').decode(buffer);
-                throw new Error(`Language.load failed with status ${response.status}.\n\n${body}`)
+                throw new Error(`Language.load failed with status ${response.status}.\n\n${body}`);
               }
             }));
       }
     }
 
     return bytes
-      .then(bytes => loadWebAssemblyModule(bytes, {loadAsync: true}))
-      .then(mod => {
-        const symbolNames = Object.keys(mod)
-        const functionName = symbolNames.find(key =>
+      .then((bytes) => loadWebAssemblyModule(bytes, {loadAsync: true}))
+      .then((mod) => {
+        const symbolNames = Object.keys(mod);
+        const functionName = symbolNames.find((key) =>
           LANGUAGE_FUNCTION_REGEX.test(key) &&
-          !key.includes("external_scanner_")
+          !key.includes('external_scanner_'),
         );
         if (!functionName) {
-          console.log(`Couldn't find language function in WASM file. Symbols:\n${JSON.stringify(symbolNames, null, 2)}`)
+          console.log(`Couldn't find language function in WASM file. Symbols:\n${JSON.stringify(symbolNames, null, 2)}`);
         }
         const languageAddress = mod[functionName]();
         return new Language(INTERNAL, languageAddress);
@@ -1059,7 +1204,7 @@ class LookaheadIterable {
   }
 
   get currentType() {
-    return this.language.types[this.currentTypeId] || 'ERROR'
+    return this.language.types[this.currentTypeId] || 'ERROR';
   }
 
   delete() {
@@ -1085,11 +1230,11 @@ class LookaheadIterable {
     return {
       next() {
         if (C._ts_lookahead_iterator_next(self[0])) {
-          return { done: false, value: self.currentType };
+          return {done: false, value: self.currentType};
         }
 
-        return { done: true, value: "" };
-      }
+        return {done: true, value: ''};
+      },
     };
   }
 }
@@ -1097,7 +1242,7 @@ class LookaheadIterable {
 class Query {
   constructor(
     internal, address, captureNames, textPredicates, predicates,
-    setProperties, assertedProperties, refutedProperties
+    setProperties, assertedProperties, refutedProperties,
   ) {
     assertInternal(internal);
     this[0] = address;
@@ -1115,15 +1260,18 @@ class Query {
     this[0] = 0;
   }
 
-  matches(node, startPosition, endPosition, options) {
-    if (!startPosition) startPosition = ZERO_POINT;
-    if (!endPosition) endPosition = ZERO_POINT;
-    if (!options) options = {};
-
-    let matchLimit = options.matchLimit;
-    if (typeof matchLimit === 'undefined') {
-      matchLimit = 0;
-    } else if (typeof matchLimit !== 'number') {
+  matches(
+    node,
+    {
+      startPosition = ZERO_POINT,
+      endPosition = ZERO_POINT,
+      startIndex = 0,
+      endIndex = 0,
+      matchLimit = 0xFFFFFFFF,
+      maxStartDepth = 0xFFFFFFFF,
+    } = {},
+  ) {
+    if (typeof matchLimit !== 'number') {
       throw new Error('Arguments must be numbers');
     }
 
@@ -1136,7 +1284,10 @@ class Query {
       startPosition.column,
       endPosition.row,
       endPosition.column,
-      matchLimit
+      startIndex,
+      endIndex,
+      matchLimit,
+      maxStartDepth,
     );
 
     const rawCount = getValue(TRANSFER_BUFFER, 'i32');
@@ -1155,14 +1306,15 @@ class Query {
 
       const captures = new Array(captureCount);
       address = unmarshalCaptures(this, node.tree, address, captures);
-      if (this.textPredicates[pattern].every(p => p(captures))) {
-        result[filteredCount++] = {pattern, captures};
+      if (this.textPredicates[pattern].every((p) => p(captures))) {
+        result[filteredCount] = {pattern, captures};
         const setProperties = this.setProperties[pattern];
-        if (setProperties) result[i].setProperties = setProperties;
+        if (setProperties) result[filteredCount].setProperties = setProperties;
         const assertedProperties = this.assertedProperties[pattern];
-        if (assertedProperties) result[i].assertedProperties = assertedProperties;
+        if (assertedProperties) result[filteredCount].assertedProperties = assertedProperties;
         const refutedProperties = this.refutedProperties[pattern];
-        if (refutedProperties) result[i].refutedProperties = refutedProperties;
+        if (refutedProperties) result[filteredCount].refutedProperties = refutedProperties;
+        filteredCount++;
       }
     }
     result.length = filteredCount;
@@ -1171,15 +1323,18 @@ class Query {
     return result;
   }
 
-  captures(node, startPosition, endPosition, options) {
-    if (!startPosition) startPosition = ZERO_POINT;
-    if (!endPosition) endPosition = ZERO_POINT;
-    if (!options) options = {};
-
-    let matchLimit = options.matchLimit;
-    if (typeof matchLimit === 'undefined') {
-      matchLimit = 0;
-    } else if (typeof matchLimit !== 'number') {
+  captures(
+    node,
+    {
+      startPosition = ZERO_POINT,
+      endPosition = ZERO_POINT,
+      startIndex = 0,
+      endIndex = 0,
+      matchLimit = 0xFFFFFFFF,
+      maxStartDepth = 0xFFFFFFFF,
+    } = {},
+  ) {
+    if (typeof matchLimit !== 'number') {
       throw new Error('Arguments must be numbers');
     }
 
@@ -1192,7 +1347,10 @@ class Query {
       startPosition.column,
       endPosition.row,
       endPosition.column,
-      matchLimit
+      startIndex,
+      endIndex,
+      matchLimit,
+      maxStartDepth,
     );
 
     const count = getValue(TRANSFER_BUFFER, 'i32');
@@ -1211,10 +1369,10 @@ class Query {
       const captureIndex = getValue(address, 'i32');
       address += SIZE_OF_INT;
 
-      captures.length = captureCount
+      captures.length = captureCount;
       address = unmarshalCaptures(this, node.tree, address, captures);
 
-      if (this.textPredicates[pattern].every(p => p(captures))) {
+      if (this.textPredicates[pattern].every((p) => p(captures))) {
         const capture = captures[captureIndex];
         const setProperties = this.setProperties[pattern];
         if (setProperties) capture.setProperties = setProperties;
@@ -1231,7 +1389,15 @@ class Query {
   }
 
   predicatesForPattern(patternIndex) {
-    return this.predicates[patternIndex]
+    return this.predicates[patternIndex];
+  }
+
+  disableCapture(captureName) {
+    const captureNameLength = lengthBytesUTF8(captureName);
+    const captureNameAddress = C._malloc(captureNameLength + 1);
+    stringToUTF8(captureName, captureNameAddress, captureNameLength + 1);
+    C._ts_query_disable_capture(this[0], captureNameAddress, captureNameLength);
+    C._free(captureNameAddress);
   }
 
   didExceedMatchLimit() {
@@ -1270,7 +1436,7 @@ function unmarshalCaptures(query, tree, address, result) {
 }
 
 function assertInternal(x) {
-  if (x !== INTERNAL) throw new Error('Illegal constructor')
+  if (x !== INTERNAL) throw new Error('Illegal constructor');
 }
 
 function isPoint(point) {
@@ -1319,25 +1485,25 @@ function unmarshalNode(tree, address = TRANSFER_BUFFER) {
 function marshalTreeCursor(cursor, address = TRANSFER_BUFFER) {
   setValue(address + 0 * SIZE_OF_INT, cursor[0], 'i32'),
   setValue(address + 1 * SIZE_OF_INT, cursor[1], 'i32'),
-  setValue(address + 2 * SIZE_OF_INT, cursor[2], 'i32')
+  setValue(address + 2 * SIZE_OF_INT, cursor[2], 'i32');
 }
 
 function unmarshalTreeCursor(cursor) {
   cursor[0] = getValue(TRANSFER_BUFFER + 0 * SIZE_OF_INT, 'i32'),
   cursor[1] = getValue(TRANSFER_BUFFER + 1 * SIZE_OF_INT, 'i32'),
-  cursor[2] = getValue(TRANSFER_BUFFER + 2 * SIZE_OF_INT, 'i32')
+  cursor[2] = getValue(TRANSFER_BUFFER + 2 * SIZE_OF_INT, 'i32');
 }
 
 function marshalPoint(address, point) {
-  setValue(address, point.row, 'i32')
-  setValue(address + SIZE_OF_INT, point.column, 'i32')
+  setValue(address, point.row, 'i32');
+  setValue(address + SIZE_OF_INT, point.column, 'i32');
 }
 
 function unmarshalPoint(address) {
   return {
     row: getValue(address, 'i32'),
-    column: getValue(address + SIZE_OF_INT, 'i32')
-  }
+    column: getValue(address + SIZE_OF_INT, 'i32'),
+  };
 }
 
 function marshalRange(address, range) {
